/*
     This file is part of GNUnet.
     (C) 2003 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @author Ronaldo Alves Ferreira
 * @author Christian Grothoff
 * @author Murali Krishna Ramanathan
 * @file applications/testbed/testbed.h
 **/
#ifndef TESTBED_TESTBED_H
#define TESTBED_TESTBED_H

#include "gnunet_core.h"

/* */
#define TESTBED_HELO_RESPONSE   0       /* peer responds with a HELO */
#define TESTBED_ADD_PEER	1	/* Add a peer to a peer connection pool			*/
#define TESTBED_DEL_PEER	2	/* Delete a peer from a peer connection pool		*/
#define TESTBED_DEL_ALL_PEERS	3	/* Delete all peers from a peer connection pool		*/
#define TESTBED_GET_HELO	4	/* Get the complete host information (ID, IP, ...)	*/
#define TESTBED_SET_TVALUE	5	/* Set trust value for a peer				*/
#define TESTBED_GET_TVALUE	6	/* Get trust value of a peer				*/
#define TESTBED_BLACKLIST	7	/* Blacklist some peers					*/
#define TESTBED_SET_IN_BW	8	/* Set the inbound bandwidth				*/
#define TESTBED_SET_OUT_BW	9	/* Set the outbound bandwidth				*/
#define TESTBED_SET_LOSS_RATE	10	/* Set the drop probability of a connection		*/
#define TESTBED_LOAD_MODULE     11      /* load a module                                        */
#define TESTBED_UNLOAD_MODULE   12      /* unload a module                                      */
#define TESTBED_undefined       13
#define TESTBED_GET_STATS	42	/* Get peer statistics					*/

/* */
typedef struct {
  CS_HEADER header;  
  unsigned int msgType;	/* The message types listed above		*/
  char data[0];		/* Value is dependent on the type field		*/
} TESTBED_CS_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
} TESTBED_undefined_MESSAGE;

/* */
typedef struct {
  TESTBED_CS_MESSAGE header;
  unsigned short proto;
  unsigned short reserved; /* for alignment */
} TESTBED_GET_HELO_MESSAGE;

/* */
typedef struct {
  TESTBED_CS_MESSAGE header;
  unsigned int in_bw;		/* Inbound bandwidth  */
  unsigned int out_bw;		/* Outbound bandwidth */
} TESTBED_SET_BW_MESSAGE;

/*
 * Same struct is also used as the reply to
 * GET_HELO.
 */
typedef struct {
  TESTBED_CS_MESSAGE header;
  HELO_Message helo;
} TESTBED_ADD_PEER_MESSAGE;

/* */
typedef struct {
  TESTBED_CS_MESSAGE header;
  HostIdentity host;
} TESTBED_DEL_PEER_MESSAGE;

/* */
typedef struct {
  TESTBED_CS_MESSAGE header;
} TESTBED_DEL_ALL_PEERS_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  HostIdentity otherPeer;
} TESTBED_GET_TVALUE_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  HostIdentity otherPeer;
  unsigned int trust;
} TESTBED_SET_TVALUE_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  HostIdentity otherPeer;
} TESTBED_BLACKLIST_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  unsigned int bps;
} TESTBED_SET_IN_BW_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  unsigned int bps;
} TESTBED_SET_OUT_BW_MESSAGE;

/* refine message definition! */
typedef struct {
  TESTBED_CS_MESSAGE header;
  unsigned int percentageLoss;
} TESTBED_SET_LOSS_RATE_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  char modulename[0];
} TESTBED_LOAD_MODULE_MESSAGE;

typedef struct {
  TESTBED_CS_MESSAGE header;
  char modulename[0];
} TESTBED_UNLOAD_MODULE_MESSAGE;

#endif
